from collections import Counter
from typing import List, Set, Tuple

import nltk

from rougek.utils.stopwords import filter_stopwords
from rougek.utils.tokenizer import get_preprocessor


def build_ngrams(tokens: List[str], ngram_n: int) -> Set[Tuple[str, ...]]:
    ngrams = list(nltk.ngrams(tokens, ngram_n))
    clean_ngrams = [tuple(filter_stopwords(ngram)) for ngram in ngrams]
    clean_ngrams = [ngrams for ngrams in clean_ngrams if len(ngrams) != 0]
    return set(clean_ngrams)


def get_overlaps(refs: List[str], ngram_n: int) -> List[Tuple[str, ...]]:
    preprocessor = get_preprocessor()
    more_than_n = 2
    set_ngrams_li = [build_ngrams(preprocessor(ref), ngram_n) for ref in refs]
    ngrams = [ngram for set_ngrams in set_ngrams_li for ngram in set_ngrams]
    return [ngram for ngram, cnt in Counter(ngrams).items() if cnt >= more_than_n]


def get_ngram_overlaps(refs: List[str], max_ngram_n: int) -> List[Tuple[str, ...]]:
    """Extract ngram overlaps between reference summaries

    Parameters
    ----------
    refs : List[str]
        List of reference summaries
    max_ngram_n : int
        Upper bound of ngram length

    Returns
    -------
    List[Tuple[str, ...]]
        List of extracted keywords
    """
    selected_ngrams = []
    vocab = []
    for n in range(max_ngram_n, 0, -1):
        ngrams = get_overlaps(refs, n)
        for ngram in ngrams:
            is_to_add = True
            for word in ngram:
                if word in vocab:
                    is_to_add = False
            if is_to_add:
                selected_ngrams.append(ngram)
                vocab += list(ngram)
    return selected_ngrams




from typing import List

from rougek._rougek import filter_stopwords, find

from rougek.utils.stopwords import filter_stopwords
from rougek.utils.tokenizer import get_preprocessor


class RougeK:
    """Compute ROUGE-K score for given hypo and keywords

    Attributes
    ----------
    preprocessor : Customized spacy-based preprocessor
    """

    def __init__(self):
        self.preprocessor = get_preprocessor()

    def __call__(self, hypo: str, kws: List[str]) -> float:
        """Computes ROUGE-K for given hypo and gold kws

        Parameters
        ----------
        hypo : str
            Target summary to evaluate
        kws : List[str]
            Gold keywords to match with summary

        Returns
        -------
        float
            ROUGE-K score
        """
        words = self.preprocessor(hypo)
        words = filter_stopwords(words)

        cnt = 0
        for kw in kws:
            if find(" ".join(words), kw):
                cnt += 1

        return cnt / len(kws)




from functools import partial
from typing import Callable, List

import spacy
from spacy.lang.char_classes import (ALPHA, ALPHA_LOWER, ALPHA_UPPER,
                                     CONCAT_QUOTES, LIST_ELLIPSES, LIST_ICONS)
from spacy.tokenizer import Tokenizer
from spacy.util import compile_infix_regex


def custom_tokenizer(spacy_nlp):
    infixes = (
        LIST_ELLIPSES
        + LIST_ICONS
        + [
            r"(?<=[0-9])[+\-\*^](?=[0-9-])",
            r"(?<=[{al}{q}])\.(?=[{au}{q}])".format(
                al=ALPHA_LOWER, au=ALPHA_UPPER, q=CONCAT_QUOTES
            ),
            r"(?<=[{a}]),(?=[{a}])".format(a=ALPHA),
            # r"(?<=[{a}])(?:{h})(?=[{a}])".format(a=ALPHA, h=HYPHENS),
            r"(?<=[{a}0-9])[:<>=/](?=[{a}])".format(a=ALPHA),
        ]
    )

    infix_re = compile_infix_regex(infixes)

    return Tokenizer(
        spacy_nlp.vocab,
        prefix_search=spacy_nlp.tokenizer.prefix_search,
        suffix_search=spacy_nlp.tokenizer.suffix_search,
        infix_finditer=infix_re.finditer,
        token_match=spacy_nlp.tokenizer.token_match,
        rules=spacy_nlp.Defaults.tokenizer_exceptions,
    )


def preprocess(nlp: Tokenizer, sent: str) -> List[str]:
    """Given a sentence, clean and return list of ngrams"""
    sent = sent.lower()
    tokens = [
        token.lemma_ if token.lemma_ != "-PRON-" else token.orth_ for token in nlp(sent)
    ]
    return tokens


def get_preprocessor() -> Callable:
    """Get preprocessor that lower and tokenize texts

    Returns
    -------
    Callable
        A callable function that takes str and return words (List[str])
    """
    spacy_nlp = spacy.load("en_core_web_sm", disable=["parser", "ner"])
    spacy_nlp.tokenizer = custom_tokenizer(spacy_nlp)
    return partial(preprocess, spacy_nlp)




import os

import click
import sienna

from rougek import RougeK
from rougek.utils.utils import avg, std


def evaluate_hypo_file(hypo_path: str, kws_li: list[list[str]]) -> list[float]:
    hypos = sienna.load(hypo_path)
    assert isinstance(hypos, list)
    assert isinstance(hypos[0], str)
    return evaluate_hypos(hypos, kws_li)


def evaluate_hypos(hypos: list[str], kws_li: list[list[str]]) -> list[float]:
    rk = RougeK()
    rk_scores: list[float] = []
    for hypo, kws in zip(hypos, kws_li):
        rk_scores.append(rk(hypo, kws))
    return rk_scores


@click.command()
@click.argument("hypo_path")
@click.argument("kws_path")
def evaluate(hypo_path: str, kws_path: str) -> None:
    """Command to evaluate summaries by ROUGE-K"""
    rk = RougeK()

    hypos = sienna.load(hypo_path)
    kws_li = sienna.load(kws_path)

    assert isinstance(hypos, list) & isinstance(hypos[0], str)
    assert isinstance(kws_li, list) & isinstance(kws_li[0], list)

    rk_scores = []

    for hypo, kws in zip(hypos, kws_li):
        rk_scores.append(rk(hypo, kws))

    print(f"Avg ROUGE-K is: {avg(rk_scores)}±{std(rk_scores)}")


@click.command()
@click.argument("hypo_dir")
@click.argument("kws_path")
def evaluate_dir(hypo_dir: str, kws_path: str) -> None:
    """Command to evaluate summaries by ROUGE-K"""
    hypo_paths = [
        os.path.join(hypo_dir, hypo_filename)
        for hypo_filename in os.listdir(hypo_dir)
        if hypo_filename.endswith(".hypo")
    ]
    
    print(f"{len(hypo_paths)} target file found.")

    kws_li = sienna.load(kws_path)
    assert isinstance(kws_li, list)
    assert isinstance(kws_li[0], list)

    hypo_scores = []
    for hypo_path in hypo_paths:
        rk_scores = evaluate_hypo_file(hypo_path, kws_li)
        hypo_scores.append(avg(rk_scores))

    print(f"Avg ROUGE-K is: {avg(hypo_scores)}±{std(hypo_scores)}")




import click

from rougek.evaluation.commands import evaluate, evaluate_dir
from rougek.extract.commands import extract


@click.group()
def cli():
    pass


cli.add_command(extract)
cli.add_command(evaluate)
cli.add_command(evaluate_dir, name="evaluate_dir")


def main():
    cli()



from typing import List

import click
import sienna
from tqdm import tqdm

from rougek.extract.utils import get_ngram_overlaps


@click.command()
@click.argument("dpath")
@click.argument("opath")
def extract(dpath: str, opath: str) -> None:
    """Command to generate a list of keywords"""
    data = sienna.load(dpath)
    assert isinstance(data, list)
    assert isinstance(data[0], dict)

    result_ngrams_li: List[List[str]] = []
    for d in tqdm(data):
        refs = d["target"]
        if len(refs):
            refs += [d["title"]]
        assert isinstance(refs, list), f"Wrong format., type(refs)=={type(refs)}"
        assert len(refs) >= 2, f"Refs too small. len(refs)=={len(refs)}"
        assert isinstance(
            refs[0], str
        ), f"Wrong format., type(refs[0])=={type(refs[0])}"

        ngrams = get_ngram_overlaps(refs, 10)
        ngrams_str = [" ".join(ngram) for ngram in ngrams]
        result_ngrams_li.append(ngrams_str)

    print(f"Saving keyword list to {opath}...")
    sienna.save(result_ngrams_li, opath)

